@@ -253,38 +253,38 @@ __asm(".text\n"
 __ALIGN_STR "\n"
 ".globl " SYMBOL_NAME_STR(startcode_beg) ";\n"
 ".globl " SYMBOL_NAME_STR(startcode_end) ";\n"
-SYMBOL_NAME_STR(startcode_beg) ":
-								| /* copy the ramdisk to the top of memory */
-								| /* (from back to front) */
-		addl	%d1,%a4			| src   = (u_long *) (rd_src + rd_size);
-		movel	%a3,%a5
-		addl	%d1,%a5			| dest  = (u_long *) (rd_dest + rd_size);
-
-		bras	2f				| do
-1:		movel	-(%a4),-(%a5)	|   *--dest = *--src;
-2:		cmpl	%a3,%a5			| while (dest > ramdisk_dest)
-		jgt		1b				| 
-
-								| /* copy kernel text and data, and bootinfo */
-		movel	%a2,%a4			| limit = (u_long *) (kernel_src + kernel_size);
-		addl	%d0,%a4
-		movel	%a1,%a5			| dest  = (u_long *) kernel_dest
-
-		bras	2f				| do
-1:		movel	(%a2)+,(%a5)+	|  *dest++ = *src++;
-2:		cmpl	%a4,%a2			| while (src < limit)
-		jlt		1b				|
-
-		tstl	%d2				| call BVMbug?
-		jeq		1f				| branch if not
-
-		movel	%a1,-(%a7)		| return to start of kernel (kernel_dest)
-		movel	#0xf8000404,%a1	| get BVMBug entry point
-		moveq	#22,%d0			| BVMBug enter
-
-1:		jmp		(%a1)			| start kernel or enter BVMBug
+SYMBOL_NAME_STR(startcode_beg) ":											\n\
+								| copy the ramdisk to the top of memory		\n\
+								| (from back to front)						\n\
+		addl	%d1,%a4			| src   = (u_long *) (rd_src + rd_size);	\n\
+		movel	%a3,%a5														\n\
+		addl	%d1,%a5			| dest  = (u_long *) (rd_dest + rd_size);	\n\
+																			\n\
+		bras	2f				| do										\n\
+1:		movel	-(%a4),-(%a5)	|   *--dest = *--src;						\n\
+2:		cmpl	%a3,%a5			| while (dest > ramdisk_dest)				\n\
+		jgt		1b				| 											\n\
+																			\n\
+								| copy kernel text and data, and bootinfo	\n\
+		movel	%a2,%a4			| limit =									\n\
+		addl	%d0,%a4			|	(u_long *) (kernel_src + kernel_size);	\n\
+		movel	%a1,%a5			| dest  = (u_long *) kernel_dest			\n\
+																			\n\
+		bras	2f				| do										\n\
+1:		movel	(%a2)+,(%a5)+	|  *dest++ = *src++;						\n\
+2:		cmpl	%a4,%a2			| while (src < limit)						\n\
+		jlt		1b				|											\n\
+																			\n\
+		tstl	%d2				| call BVMbug?								\n\
+		jeq		1f				| branch if not								\n\
+																			\n\
+		movel	%a1,-(%a7)		| return to start of kernel (kernel_dest)	\n\
+		movel	#0xf8000404,%a1	| get BVMBug entry point					\n\
+		moveq	#22,%d0			| BVMBug enter								\n\
+																			\n\
+1:		jmp		(%a1)			| start kernel or enter BVMBug				\n\
 "
-SYMBOL_NAME_STR(startcode_end) ":
+SYMBOL_NAME_STR(startcode_end) ":											\n\
 ");
 
 /*--------------------------------------------------------------------------*/
