@@ -18,8 +18,8 @@
 __asm__(".text\n"
 __ALIGN_STR "\n"
 ".globl " SYMBOL_NAME_STR(_start) ";\n"
-SYMBOL_NAME_STR(_start) ":
-	jbra	(" SYMBOL_NAME_STR(Main) ");
-");
+SYMBOL_NAME_STR(_start) ":\n"
+"	jbra	(" SYMBOL_NAME_STR(Main) ");\n"
+);
 
 /*-----------------------------< end of file >------------------------------*/
