@@ -879,7 +879,7 @@ load_symbols
 	}
 
 done:
-	percent_term("OK\n");
+	progress_term("OK\n");
 
 fail:
 	sclose();
@@ -1379,8 +1379,8 @@ load_kernel
 			printf("Kernel segment %u at 0x%08lx, size %lu, align %lu\n",
 				i,
 				mem_start + kernel_phdrs[i].p_vaddr - PAGE_SIZE,
-				kernel_phdrs[i].p_memsz,
-				kernel_phdrs[i].p_align
+				(unsigned long) kernel_phdrs[i].p_memsz,
+				(unsigned long) kernel_phdrs[i].p_align
 			);
 		}
 
@@ -1400,11 +1400,11 @@ load_kernel
 
  	if (v->debug_mode)
 	{
-		printf("Kernel entry is 0x%08lx.\n", kexec_elf.e_entry);
+		printf("Kernel entry is 0x%08lx.\n", (unsigned long) kexec_elf.e_entry);
 	}
 
 	sclose();
-	percent_term("OK\n");
+	progress_term("OK\n");
 
 	free(kernel_phdrs);
 
@@ -1611,7 +1611,7 @@ boot_linux
 		}
 
 		sclose();
-		percent_term("OK\n");
+		progress_term("OK\n");
 
 		/* make sure ramdisk will sit on a 1K boundry at end of memory */
 		v->bi.ramdisk.addr = mem_end - ((v->bi.ramdisk.size + 1023) & ~1023);
@@ -1670,8 +1670,14 @@ boot_linux
    	/* append boot record command line arguments */
 	if (boot_record->cmdline)
 	{
-		strcatn(v->bi.command_line, " ",                  CL_SIZE);
-   		strcatn(v->bi.command_line, boot_record->cmdline, CL_SIZE);
+		expand_bootrom_args(v->input_buffer, boot_record->cmdline,
+				sizeof(v->input_buffer));
+
+		if (v->input_buffer[0])
+		{
+			strcatn(v->bi.command_line, " ",             CL_SIZE);
+   			strcatn(v->bi.command_line, v->input_buffer, CL_SIZE);
+   		}
 	}
 
    	/* append user specified input */
@@ -1684,8 +1690,14 @@ boot_linux
    	/* append any additional arguments */
 	if (boot_record->append)
 	{
-		strcatn(v->bi.command_line, " ",                 CL_SIZE);
-   		strcatn(v->bi.command_line, boot_record->append, CL_SIZE);
+		expand_bootrom_args(v->input_buffer, boot_record->append,
+				sizeof(v->input_buffer));
+
+		if (v->input_buffer[0])
+		{
+			strcatn(v->bi.command_line, " ",             CL_SIZE);
+   			strcatn(v->bi.command_line, v->input_buffer, CL_SIZE);
+   		}
    	}
 
    	if (v->debug_mode)
