@@ -30,6 +30,25 @@
 
 #include "vmelilo.h"
 
+/*--------------------------------------------------------------------------*/
+/* Print a Message unless --quiet option specified.
+ */
+
+void
+message
+(	const char	*fmt,
+	...
+)
+{	va_list		ap;
+
+	if (!f_quiet)
+	{
+		va_start(ap, fmt);
+		vfprintf(stdout, fmt, ap);
+		va_end(ap);
+	}
+}
+
 /*--------------------------------------------------------------------------*/
 /* Print an Error Message and Exit
  */
@@ -42,9 +61,11 @@ die
 {	va_list		ap;
 
 	fflush(stdout);
+	fputc('\n', stderr);
 	va_start(ap, fmt);
 	vfprintf(stderr, fmt, ap);
 	va_end(ap);
+	fputc('\n', stderr);
 
 	exit(1);
 }
@@ -146,13 +167,13 @@ write_file
 
 	if (f_test)
 	{
-		printf("Not writing file `%s'\n", name);
+		message("Test mode - file `%s' not written.\n", name);
 		return;
 	}
 
 	if (f_verbose)
 	{
-		printf("Writing file `%s'\n", name);
+		message("Writing file `%s'.\n", name);
 	}
 
 	if ((fh = open(name, O_CREAT|O_RDWR|O_TRUNC, S_IRUSR|S_IWUSR)) == -1)
@@ -181,7 +202,7 @@ read_boot_block
 
 	if (f_verbose)
 	{
-		printf("Reading boot block from `%s'\n", name);
+		message("Reading boot block from `%s'.\n", name);
 	}
 
 	if ((fh = open(name, O_RDONLY)) == -1)
@@ -207,50 +228,58 @@ write_boot_block
 (	const char		*name
 )
 {	int				fh;
-
-	if (f_verbose)
-	{
-		printf("%sriting boot block to device `%s'\n",
-			f_test ? "Not w" : "W", name);
-	}
+	int				i;
+	int				j;
 
 	if (f_test)
 	{
-		int i,j;
-		unsigned char *cp = (unsigned char *)boot_block_buffer;
+		unsigned char	*cp = (unsigned char *)boot_block_buffer;
+
+		message("Test mode - boot block for device %s - not written.\n",
+			name);
+
+		if (f_install)
+			message("Boot loader file not yet written, boot block inaccurate.\n");
 
-		printf ("Test mode - boot block is 0x%04x bytes - not written\n",
-			boot_block_size);
 		for (i = 0; i < boot_block_size; i += 16, cp += 16)
 		{
 			int max = boot_block_size - i;
 			if (max > 16) max = 16;
-			printf("%03X:", i);
+			message("%03X:", i);
 			for (j = 0; j < 16; j++)
 			{
-				if ((j & 1) == 0) printf(" ");
-				if (j < max) printf("%02X", cp[j]);
-				else printf("  ");
+				if ((j & 1) == 0)
+					message(" ");
+				if (j < max)
+					message("%02X", cp[j]);
+				else
+					message("  ");
 			}
-			printf("  ");
+			message("  ");
 			for (j = 0; j < max; j++)
-				printf("%c", (cp[j] < ' ' || cp[j] > '~') ? '.' : cp[j]);
-			printf("\n");
+				message("%c", (cp[j] < ' ' || cp[j] > '~') ? '.' : cp[j]);
+			message("\n");
 		}
-		return;
 	}
-
-	if ((fh = open(name, O_RDWR)) == -1)
+	else
 	{
-		error_open(name);
+		if (f_verbose)
+		{
+			message("Writing boot block to device `%s'.\n", name);
+		}
+	
+		if ((fh = open(name, O_RDWR)) == -1)
+		{
+			error_open(name);
+		}
+	
+		if (write(fh, boot_block_buffer, boot_block_size) != boot_block_size)
+		{
+			error_write(name);
+		}
+	
+		close(fh);
 	}
-
-	if (write(fh, boot_block_buffer, boot_block_size) != boot_block_size)
-	{
-		error_write(name);
-	}
-
-	close(fh);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -269,12 +298,12 @@ backup_boot_block
 		write_file(backup_file, boot_block_buffer, boot_block_size);
 		if (f_verbose)
 		{
-	    	printf("Backup copy of boot sector in `%s'\n", backup_file);
+	    	message("Backup copy of boot sector in `%s'.\n", backup_file);
 	    }
 	}
 	else if (f_verbose)
 	{
-	    printf("`%s' exists - no backup copy made.\n", backup_file);
+	    message("`%s' exists - no backup copy made.\n", backup_file);
 	}
 }
 
@@ -406,7 +435,7 @@ write_tag_string
  */
 
 static
-const char *
+char *
 create_file_name
 (	const char	*name
 )
@@ -438,6 +467,69 @@ create_file_name
 	return s;
 }
 
+/*--------------------------------------------------------------------------*/
+
+static
+int
+check_file_access
+(	const char		*path
+)
+{	int				ret;
+	char			*fullpath;
+
+	fullpath = create_file_name(path);
+	ret      = access(fullpath, R_OK) == 0;
+	free(fullpath);
+
+	return ret;
+}
+
+/*--------------------------------------------------------------------------*/
+/* Return FALSE if 'path' exists.
+ * Return TRUE  if 'path' does not exist but boot record is optional.
+ * otherwise die().
+ */
+
+static
+int
+die_unless_optional
+(	const BOOTRECORD	*record,
+	const char			*path,
+	const char			*description
+)
+{	int					optional;
+
+	if (check_file_access(path))
+	{
+		/* exists */
+		return FALSE;
+	}
+
+	optional = record->optional
+			 ? *record->optional
+			 : FALSE;
+
+	if (!optional)
+	{
+		/* does not exist, but not optional */
+		die("%s - %s file '%s' is not available.\n",
+					record->label, description, path);
+	}
+	
+	if (f_verbose)
+	{
+		message("   %-32.32s -> ** Skipped **\n", record->label);
+	}
+
+	/* dereference files we are not going to use */
+	remove_file_def(with_default(record, kernel));
+	remove_file_def(record->ramdisk             );
+	remove_file_def(record->symtab              );
+
+	/* does not exist, but optional */
+	return TRUE;
+}
+
 /*--------------------------------------------------------------------------*/
 /* Create the Map Data
  */
@@ -458,6 +550,7 @@ create_map_data
 	const char			*root;
 	const u_long		*read_only;
 	const u_long		*restricted;
+	int					records = 0;
 
 #define WRITE_TAG_STRING(f, tag, arg) \
 	do	{if (arg) write_tag_string(f, tag, arg);} while(0)
@@ -485,13 +578,18 @@ create_map_data
 	/* boot records */
 	if (f_verbose)
 	{
-		printf("[Boot Records]\n");
+		message("[Boot Records]\n");
 	}
 	for (record = config.records; record; record = record->next)
 	{
-		password	= record->password
-					? record->password
-					: config.default_password;
+		password	= with_default(record, password  );
+		restricted	= with_default(record, restricted);
+		cmdline		= with_default(record, cmdline   );
+		append		= with_default(record, append    );
+		kernel		= with_default(record, kernel    );
+		console		= with_default(record, console   );
+		root		= with_default(record, root      );
+		read_only	= with_default(record, read_only );
 
 		/* don't write a zero lenth password */
 		if (password && (*password == '\0'))
@@ -499,46 +597,40 @@ create_map_data
 			password = NULL;
 		}
 
-		restricted	= record->restricted
-					? record->restricted
-					: config.default_restricted;
-
 		/* don't write restricted flag if FALSE */
 		if (restricted && (*restricted == FALSE))
 		{
 			restricted = NULL;
 		}
 
-		cmdline		= record->cmdline
-					? record->cmdline
-					: config.default_cmdline;
-		
-		append		= record->append
-					? record->append
-					: config.default_append;
-
-		kernel		= record->kernel
-					? record->kernel
-					: config.default_kernel;
-
-		console		= record->console
-					? record->console
-					: config.default_console;
-
-		root		= record->root
-					? record->root
-					: config.default_root;
-
-		read_only	= record->read_only
-					? record->read_only
-					: config.default_read_only;
-
-		/* must have a kernel */
+		/* must specify a kernel */
 		if ((kernel == NULL) || (*kernel == '\0'))
 		{
 			die("No kernel image specified for `%s'\n", record->label);
 		}
 
+		/* does kernel image exist */
+		if (die_unless_optional(record, kernel, "kernel image"))
+		{
+			continue;
+		}
+	
+		if (record->ramdisk && *record->ramdisk)
+		{
+			if (die_unless_optional(record, record->ramdisk, "ramdisk image"))
+			{
+				continue;
+			}
+		}
+
+		if (record->symtab && *record->symtab)
+		{
+			if (die_unless_optional(record, record->symtab, "kernel symbol"))
+			{
+				continue;
+			}
+		}
+
 		/* find default boot record as we go */
 		if (equal_strings(config.options.boot_default, record->label)
 		||	equal_strings(config.options.boot_default, record->alias))
@@ -546,8 +638,23 @@ create_map_data
 			defrecord = record;
 		}
 
-		printf(f_verbose ? "  %c%-32.32s -> %s\n" : "  %c%s\n",
-			(defrecord == record) ? '*' : ' ', record->label, kernel);
+		if (f_verbose)
+		{
+			message("  %c%-32.32s -> %s\n",
+				(defrecord == record) ? '*' : ' ',
+				record->label,
+				kernel);
+			if (record->ramdisk && *record->ramdisk)
+				message("%36s-> %s\n", "", record->ramdisk);
+			if (record->symtab && *record->symtab)
+				message("%36s-> %s\n", "", record->symtab);
+		}
+		else
+		{
+			message("  %c%s\n",
+				(defrecord == record) ? '*' : ' ',
+				record->label);
+		}
 
 		write_tag_string(&map_data, TAG_BOOT_RECORD,     record->label        );
 		WRITE_TAG_STRING(&map_data, TAG_ALIAS,           record->alias        );
@@ -564,39 +671,53 @@ create_map_data
 		WRITE_TAG_STRING(&map_data, TAG_CONSOLE,         console              );
 		WRITE_TAG_DATA  (&map_data, TAG_READ_ONLY,       read_only,          4);
 		write_tag_data  (&map_data, TAG_BOOT_RECORD_END, NULL,               0);
+		records++;
+	}
+
+	if (records == 0)
+	{
+		die("There are no bootable configurations.\n");
 	}
 
 	if (!defrecord)
 	{
-		die("Default boot record `%s' does not exist\n",
+		die("Default boot record `%s' does not exist.\n",
 			config.options.boot_default);
 	}
 
 	if (f_verbose)
 	{
-		printf("[File Definitions]\n");
+		message("[File Definitions]\n");
 	}
 	for (file = config.files; file; file = file->next)
 	{
-		const char *path = create_file_name(file->path);
+		char	*path;
+
+		/* skip file if it is not in any active configuration */
+		if (file->refcount == 0)
+		{
+			continue;
+		}
+
+		path = create_file_name(file->path);
 
 		if ((file->map = create_file_map(path)) == NULL)
 		{
 			if (case_equal_strings(defrecord->kernel, file->path))
 			{
 				die("Kernel image `%s' for default boot record `%s' does not "
-						"exist\n", path, defrecord->label);
+						"exist.\n", path, defrecord->label);
 			}
 			else
 			{
-				printf("Warning: file `%s' doesn't exist\n", path);
+				message("Warning: file `%s' doesn't exist.\n", path);
 			}
 		}
 		else
 		{
 			if (f_verbose)
 			{
-				printf("  %s (%ld bytes in %ld fragments)\n",
+				message("  %s (%ld bytes in %ld fragments)\n",
 						path, MAP_FILESIZE(file->map), MAP_NUMFRAGS(file->map));
 			}
 
@@ -605,6 +726,8 @@ create_map_data
 												MAP_MAPSIZE(file->map));
 			write_tag_data  (&map_data, TAG_FILE_DEF_END, NULL, 0);
 		}
+
+		free(path);
 	}
 
 	/* end of file */
@@ -612,7 +735,7 @@ create_map_data
 
 	if (!f_verbose)
 	{
-		putchar('\n');
+		message("\n");
 	}
 }
 
@@ -641,14 +764,15 @@ write_loader
 		}
 	}
 
-	die("Couldn't find magic key in loader template\n");
+	die("Couldn't find magic key in loader template.\n");
 
 found_it:
 
 	if (f_verbose)
 	{
-		printf("%sriting loader to file `%s'\n",
-			f_test ? "Not w" : "W", loader_file);
+		message(f_test ? "Test mode - loader file '%s' - not written.\n"
+					  : "Writing loader to file `%s'.\n",
+				 loader_file);
 	}
 
 	if (f_test)
@@ -746,7 +870,7 @@ get_m68k_model
 			case MODEL_BVME: model = "BVM BVME4000/6000"; break;
 			case MODEL_MVME: model = "Motorola MVME SBC"; break;
 		}
-		printf("VME Architecture: %s\n", model);
+		message("VME Architecture: %s\n", model);
 	}
 }
 
@@ -766,6 +890,7 @@ usage
 "Valid options are:\n"
 "    -h, --help                Display this usage information\n"
 "    -v, --verbose             Enable verbose mode\n"
+"    -q, --quiet               Operate silently\n"
 "    -a, --arch id             Force use of specific boot loader,\n"
 "                              where id is: bvme or mvme\n"
 "                               bvme - BVME4000/6000\n"
@@ -793,7 +918,7 @@ parse_opts
 (	int			argc,
 	char		*argv[]
 )
-{	static const char			s_opts[] = "BVa:b:c:d:fhl:r:s:tuvw:"; 
+{	static const char			s_opts[] = "BVa:b:c:d:fhl:qr:s:tuvw:"; 
 	static const struct option	l_opts[] = {
 		{"force-backup",		no_argument,			NULL,	'B'},
 		{"version",				no_argument,			NULL,	'V'},
@@ -804,6 +929,7 @@ parse_opts
 		{"force",				no_argument,			NULL,	'f'},
 		{"help",				no_argument,			NULL,	'h'},
 		{"loader-file",			required_argument,		NULL,	'l'},
+		{"quiet",				no_argument,			NULL,	'q'},
 		{"restore-from",		required_argument,		NULL,	'r'},
 		{"save-to",				required_argument,		NULL,	's'},
 		{"test",				no_argument,			NULL,	't'},
@@ -932,6 +1058,12 @@ parse_opts
 				break;
 			}
 
+			case 'q':
+			{
+			    f_quiet = TRUE;
+				break;
+			}
+
 			case 'w':
 			{
 				if (root_path) goto duplicate_option;
@@ -968,7 +1100,7 @@ parse_opts
 
     if (f_verbose)
     {
-		printf(LiloVersion);
+		message(LiloVersion);
 	}
 
 	/* if no configuration file specified */
@@ -998,7 +1130,7 @@ parse_opts
 	{
         if (f_verbose)
         {
-            printf("Using specified device `%s'\n", device);
+            message("Using specified device `%s'.\n", device);
         }
 
         config.boot_device_name = device;
@@ -1071,29 +1203,29 @@ main
     {
 	    if (!f_force && !valid_boot_block(boot_block_buffer))
 	    {
-		    printf("%s has a non %s boot sector\n",
+		    message("%s has a non %s boot sector.\n",
 					config.boot_device_name, monitor_name);
-	    	die("Use --force option to continue\n");
+	    	die("Use --force option to continue.\n");
 		}
     }
 
     if (save_boot_block)
 	{
 		write_file(save_boot_block, boot_block_buffer, boot_block_size);
-		printf("Boot block saved to file `%s'\n", save_boot_block);
+		message("Boot block saved to file `%s'.\n", save_boot_block);
     }
 
     else if (restore_boot_block)
 	{
 		read_boot_block(restore_boot_block);
-		printf("Boot block read from file `%s'\n", restore_boot_block);
+		message("Boot block read from file `%s'.\n", restore_boot_block);
 		write_boot_block(config.boot_device_name);
     }
 
     else if (f_uninstall)
 	{
 		read_boot_block(backup_file);
-		printf("Original boot block read from file `%s'\n", backup_file);
+		message("Original boot block read from file `%s'.\n", backup_file);
 		write_boot_block(config.boot_device_name);
     }
 
@@ -1105,12 +1237,15 @@ main
 
 		if ((loader_map = create_file_map(loader_file)) == NULL)
 		{
-		    error_open(loader_file);
+			if (!f_test)
+			{
+				error_open(loader_file);
+			}
 		}
 
 		if (f_verbose)
 		{
-			puts("Creating boot block");
+			puts("Creating boot block.");
 		}
 
 		if (!valid_boot_block(boot_block_buffer))
