@@ -14,6 +14,9 @@
 #include <sys/types.h>
 
 #include "version.h"
+
+#define CONFIG_PARSER	1		/* include extra vmelilo config parser stuff*/
+
 #include "loaderdefs.h"
 
 /*--------------------------------------------------------------------------*/
@@ -42,6 +45,9 @@ typedef struct {
 
 } CONFIG;
 
+#define with_default(rec, mem) \
+  (((rec)->mem != NULL) ? (rec)->mem : config.default_ ## mem)
+
 /*--------------------------------------------------------------------------*/
 
 typedef struct {
@@ -82,6 +88,7 @@ GLOBAL const char	LiloVersion[]			_I_( = VERSION );
  */
 GLOBAL const char		*program_name;
 GLOBAL int				f_verbose;
+GLOBAL int				f_quiet;
 GLOBAL int				f_install;
 GLOBAL int				f_backup;
 GLOBAL int				f_uninstall;
@@ -140,6 +147,7 @@ char * skip_white(char *);
 int equal_strings (const char *s1, const char *s2);
 int case_equal_strings (const char *s1, const char *s2);
 void die (const char *fmt, ...) __attribute__ ((noreturn, format (printf, 1, 2)));
+void message(const char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
 void error_nomemory (void);
 void error_open (const char *name);
 void error_opendir (const char *name);
@@ -152,5 +160,6 @@ void read_config_file (void);
 int is_scsi_partition (unsigned rdev);
 const FILEMAP * create_file_map (const char *name);
 const char * create_backup_filename (void);
+void remove_file_def(const char *path);
 
 /*-----------------------------< end of file >------------------------------*/
